%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Network growth model with activity-dependent neurite outgrowth and migration
%
% Publication:
% Okujeni S, Egert U. 2019.
% Self-organization of modular network architecture by activity-dependent neuronal migration and outgrowth.
% Elife 8. doi:10.7554/eLife.47996
%
%  Samora Okujeni 2018
%
% The model is based on the model described in van Ooyen et al. 1995 (doi:10.1006/jtbi.1995.0005).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ------------------------------------------------------------------------------
%                               parameters
% network properties
network_size                    = [1 1];                  % default = [1 1]; network extent [mm] (network is placed on a torus)
min_distance                    = 0.012;                  % default = 0.012; minimally allowed distance between neurons (two times their radius)
n_neurons                       = 500;                    % default = 500; number of neurons in the network

% neuron properties
E_min                           = 0;                      % default = 0; minimal membrane potential
E_max                           = 1;                      % default = 1; maximal membrane potential
params_activity                 = [E_min E_max 0.5 0.12]; % default = [E_min E_max 0.5 0.12]; sigmoid parameters for the input to activity transfer function [E_min E_max -1 theta alpha]
params_growth                   = [1 -1 0.6 0.1];         % default = [1 -1 0.6 0.1]; sigmoid parameters for the activity to growth transfer function [1 -1 epsilon beta]
params_migration                = -15;                    % default = -15; negative exponential function for migration depending on activity

% scaling factor for connectivity
s = 0.1;                                                  % default = 0.1; weighting factor for synapse determines the amount of overlap in mature networks

% neurite growth parameters
Rho_migration                   = 0.3;                    % default = 0/0.01/0.05/0.3; migration rate per day in mm
Rho_growth                      = 0.004;                  % default = 0.004;  neurite field expansion rate per day in mm (for sparse networks of e.g. 100 neurons neurite fields extend less radially but grow long and less branched. This would could be simulated with a higher growth rate (e.g. ~0.007)).
radius_start                    = min_distance/2;         % neurite radius at 0 DIV (cell body radius)

% random movement component and jitter
p_random                        = .9;                     % default = .9; relative weigth of random movement component
t_random                        = 10;                     % default = 10; duration [min] for which the direction of random movement component is maintained the same
D_jitter                        = min_distance/4;         % jitter distance per time step(half the cell body radius)

% simulation
dt                              = 1;                      % default = 1; time resolution [min]
div_max                         = 50;                     % default = 50; end of simulation [days] 
plot_interval                   = 1;                      % default = 1; time interval to update plot [hours]: 

%% -------------------------------------------------------------------------------------------------------------
% display growth and migration functions
figure, hold on;
plot(0:0.01:1,exp([0:0.01:1]*params_migration));
plot(0:0.01:1,fcnSigmoidal(0:0.01:1,params_growth),'r');
plot([0 1],[0 0],'k');
plot([params_growth(3) params_growth(3)],[-1 1],'k');
set(gca,'TickDir','out'); axis square
figure, hold on;
plot(0:0.01:1,fcnSigmoidal(0:0.01:1,params_activity));
plot([0 1],[params_growth(3) params_growth(3)],'k');
set(gca,'TickDir','out'); axis square

%% -------------------------------------------------------------------------------------------------------------
% simulation time vector
ts_day                          = 60*24/dt;               % number of time steps per day
time_bins = [0:dt:div_max*ts_day]/ts_day;                 % time bins in days

%% -----------------------------------------------------------------------------------------------------
% initialize structure carrying the state of all neurons (neurons)

% radius
neurons.radius = radius_start+zeros(n_neurons,1);

% membrane potential
neurons.mp     = ones(n_neurons,1)*E_min;

% position (place neurons on torus)
neurons.position = rand(1,2).*network_size;
while size(neurons.position,1)<n_neurons
    new_position = rand(1,2).*network_size;
    dxy = abs(neurons.position-repmat(new_position,size(neurons.position,1),1));
    dxy = min(cat(3,repmat(network_size,size(neurons.position,1),1)-dxy,dxy),[],3);
    d = min(sqrt(sum(dxy.^2,2)));
    if d >= min_distance
        neurons.position = [neurons.position;new_position];
    end
end

% membrane potential
neurons.connectivity = zeros(n_neurons,1);

% activity
neurons.activity     = zeros(n_neurons,1);

% display minimal neuron distance and CI
ci = getCI_bootstrap_torus(neurons.position,network_size,min_distance,1000);
D = getDistanceTorus(neurons.position, network_size); D(find(eye(size(D))))=inf;
disp(['min distance=' num2str(min(D(:))*1e3,4) char(181) 'm; CI=' num2str(ci)]);

%% -----------------------------------------------------------------------------------------------------
% initialize structure to store the whole simulation

sim = struct();
sim.positions          = nan(length(time_bins),n_neurons,2);
sim.positions(1,:,:)   = neurons.position;
sim.radius             = nan(length(time_bins),n_neurons);
sim.connectivity       = nan(length(time_bins),n_neurons);
sim.mp                 = nan(length(time_bins),n_neurons);
sim.activity           = nan(length(time_bins),n_neurons);
sim.overlap            = nan(length(time_bins),n_neurons);
sim.input              = nan(length(time_bins),n_neurons);
sim.CI                 = nan(size(time_bins));

%% -----------------------------------------------------------------------------------------------------
% initialize plot for simulation

nd_max = 25; % maximal neurite density displayed

figure, hold on;
% neurite density image
ds =.001; % pixel resolution of the image (mm)
ax = 0:ds:network_size(1);
ay = 0:ds:network_size(2);
Ax = meshgrid(ax);
Ay = meshgrid(ay)';
A = zeros(size(Ax));
ha_img = imagesc(ax,ay,A);
colmap = [[0 0 0];parula(255)];
colormap(colmap);
set(gca,'CLim',[0 nd_max]);
% neurite field outlines
ang = 0:0.01:2*pi;
xval = [repmat(neurons.position(:,1),1,length(ang))+repmat(cos(ang),n_neurons,1).*repmat(neurons.radius,1,length(ang)),nan(n_neurons,1)]';
yval = [repmat(neurons.position(:,2),1,length(ang))+repmat(sin(ang),n_neurons,1).*repmat(neurons.radius,1,length(ang)),nan(n_neurons,1)]';
xval = mod(xval+network_size(1),network_size(1));
yval = mod(yval+network_size(1),network_size(1));
id = find(sqrt(diff(xval(:)).^2+diff(yval(:)).^2)>(2*pi*sqrt(prod(network_size)))/length(ang));
xval(id)= nan;
yval(id)= nan;
ha_circle = plot(xval(:),yval(:),'-','Color',[1 1 1]);
% neurite positions
ha_position  = plot(neurons.position(:,1),neurons.position(:,2),'wo','MarkerFaceColor','r');
axis equal;
axis([0 network_size(1) 0 network_size(2)]);
old_jitter =  zeros(n_neurons,2);
% time display
ha_div = text(network_size(2)/40,network_size(1)-network_size(1)/40,[num2str(0), ' DIV ' num2str(0) 'h'],'FontWeight','bold','FontSize',20,'HorizontalAlignment','left','interp','none','Color','w');
xlabel('x [mm]');
ylabel('y [mm]');
% modularity display
% C = neurons.connectivity;
% C(find(eye(size(C)))) = 0;
% [~, Q]     = community_louvain(C,1);
% ha_Q = text((35*network_size(2))/40,network_size(1)-network_size(1)/40,['Q=' num2str(Q,2)],'FontWeight','bold','FontSize',20,'HorizontalAlignment','left','interp','none','Color','w');
set(gcf,'position',[50 50 1000 1000]);
%% -----------------------------------------------------------------------------------------------------
%                               simulate
for dd = 1:length(time_bins),
    % determine distance between neurons
    distmat = getDistanceTorus(neurons.position,network_size);
    
    % determine overlap of neurite fields and neurons.connectivity
    overlap = getCirclesIntersection(neurons.radius,distmat);
    
    % connectivity: normalize overlap with presynaptic neuron and multiply with weight factor s
    neurons.connectivity = overlap./(repmat(pi*neurons.radius.^2,1,n_neurons)')*s;
    
    % neuronal activity (output)
    neurons.activity = fcnSigmoidal(neurons.mp,params_activity);
    
    % input
    input_current  = dt*repmat(E_max-neurons.mp,1,n_neurons).*repmat(neurons.activity',n_neurons,1).*neurons.connectivity;
    
    % update neuronal membrane potential
    neurons.mp = neurons.mp-neurons.mp/exp(1)+nansum(input_current,2);
    
    % keep membrane potential within the limits
    neurons.mp(neurons.mp>E_max) = E_max;
    neurons.mp(neurons.mp<E_min) = E_min;
    
    % -------------------------------------------------------------------------------------------------------------
    % Neurite growth: update neurite fields
    
    impulse_growth = dt*Rho_growth/ts_day*fcnSigmoidal(neurons.activity,params_growth);
    neurons.radius =  neurons.radius+impulse_growth;
    neurons.radius(neurons.radius<min_distance/2)= min_distance/2;
    
    % -------------------------------------------------------------------------------------------------------------
    % Neuronal migration: update neuron positions
    
    % migrate
    old_position = neurons.position;     % backup old position
    direction = -getDirectionTorus(neurons.position,network_size);
    direction = direction./abs(direction);
    
    % weight direction by input
    direction = input_current.*(real(direction))+input_current.*(imag(direction))*i;%
    direction = nansum(direction,2);       % normalized direction
    direction = direction./abs(direction); % normalize
    direction(isnan(direction)) = 0;
    
    % add erratic component
    if mod(time_bins(dd),t_random/24/60)==0, % update random direction
        dir_random = (randn(n_neurons,2));
        dir_random = dir_random(:,1)+dir_random(:,2)*i;
        dir_random = dir_random./abs(dir_random);
    end
    % total migration impulse
    impulse_migration = dt*exp(neurons.activity*params_migration);
    
    % jitter
    dir_jitter = randn(n_neurons,2);
    dir_jitter = dir_jitter(:,1)+dir_jitter(:,2)*i;
    dir_jitter = dir_jitter./abs(dir_jitter);
    impulse_jitter = [real(dir_jitter),imag(dir_jitter)].*repmat(impulse_migration,1,2)*D_jitter;
    % update neuron positions
    neurons.position = neurons.position+...
        [real(direction),imag(direction)].*repmat(impulse_migration,1,2)*(1-p_random)*Rho_migration/ts_day+... % not normalized
        [real(dir_random),imag(dir_random)].*repmat(impulse_migration,1,2)*p_random*Rho_migration/ts_day+...
        impulse_jitter-old_jitter; % reset and add new jitter
    old_jitter = impulse_jitter;
    
    % keep neurons on torus
    neurons.position(neurons.position(:,1)<0,1) = network_size(1)+neurons.position(neurons.position(:,1)<0,1); % torus
    neurons.position(neurons.position(:,2)<0,2) = network_size(2)+neurons.position(neurons.position(:,2)<0,2); % torus
    neurons.position(:,1) = mod(neurons.position(:,1),network_size(1));
    neurons.position(:,2) = mod(neurons.position(:,2),network_size(2));
    
    % new distances
    distmat = getDistanceTorus(neurons.position,network_size);
    
    % discard movements conflicting with min_distance
    id = find(nanmin(distmat,[],2)<min_distance);
    neurons.position(id,:) = old_position(id,:); % revert to old position
    
    % -------------------------------------------------------------------------------------------------------------
    % save simulation
    sim.positions(dd,:,:)  = neurons.position;
    sim.radius(dd,:)       = neurons.radius;
    sim.connectivity(dd,:) = nansum(neurons.connectivity,2);
    sim.mp(dd,:)           = neurons.mp;
    sim.activity(dd,:)     = neurons.activity;
    sim.overlap(dd,:)      = nansum(overlap,2);
    sim.input(dd,:)        = nansum(input_current,2);
    sim.neuritefieldarea   = pi*sim.radius.^2;
    % -------------------------------------------------------------------------------------------------------------
    % Update plot
    if mod(time_bins(dd),plot_interval/24)==0 || time_bins(dd)==div_max*ts_day
        % update neurite density image
        A = zeros(length(ax),length(ay));
        for ii=1:n_neurons
            dx = abs(sim.positions(dd,ii,1)-Ax); dx = min(dx,network_size(1)-dx);
            dy = abs(sim.positions(dd,ii,2)-Ay); dy = min(dy,network_size(2)-dy);
            d = sqrt(dx.^2+dy.^2);
            id = find(d<sim.radius(dd,ii));
            A(id) = A(id)+1;
        end
        set(ha_img,'CData',A);
        % update neuron positions
        set(ha_position,'xdata',neurons.position(:,1),'ydata',neurons.position(:,2));
        % update neurites
        xval = [repmat(neurons.position(:,1),1,length(ang))+repmat(cos(ang),n_neurons,1).*repmat(neurons.radius,1,length(ang)),nan(n_neurons,1)]';
        yval = [repmat(neurons.position(:,2),1,length(ang))+repmat(sin(ang),n_neurons,1).*repmat(neurons.radius,1,length(ang)),nan(n_neurons,1)]';
        xval = mod(xval+network_size(1),network_size(1));
        yval = mod(yval+network_size(1),network_size(1));
        id = find(sqrt(diff(xval(:)).^2+diff(yval(:)).^2)>(2*pi*sqrt(prod(network_size)))/length(ang));
        xval(id)= nan;
        yval(id)= nan;
        set(ha_circle,'XData',xval(:),'YData',yval(:));
        % update time display
        set(ha_div,'String',[num2str(floor(time_bins(dd))), ' DIV ' num2str(mod(time_bins(dd)*24,24)) 'h']);
%         % update modularity display
%         C = neurons.connectivity;
%         C(find(eye(size(C)))) = 0;
%         [M, Q]     = community_louvain(C,1);
%         set(ha_Q,'String',['Q=' num2str(Q,2)]);
        drawnow;
    end
end

%% ------------------------------------------
% CI statistics - takes a while

reps = 1; % number of repetitions for the generation of random cell body distributions. Can be kept low since CI curves are smoothend by sliding average.
for dd=1:length(time_bins),
    sim.CI(dd)     = getCI_bootstrap_torus(squeeze(sim.positions(dd,:,:)),network_size,min_distance,reps);
end

%% -------------------------------------------------------------------------------------------------------------
% plot simulation summary


% color scheme used in paper
if Rho_migration==0
    col = [.7 0.7 0.7];
elseif Rho_migration<=0.03
    col = [1 0 0];
elseif Rho_migration<=0.1
    col = [0 0 0];
elseif Rho_migration<=0.5
    col = [0 0.5 0];
else
    col = [0 0 1];
end


figure
% neurite field area (mm2)
subplot(3,2,1);hold on;
plot(time_bins,nanmean(sim.neuritefieldarea,2),'Color',col);
set(gca,'XLim',[0 max(time_bins)],'YLim',[0 0.02],'TickDir','out');
xlabel('DIV'); ylabel('neurite field area [mm^2]');

% migration distance (mm)
subplot(3,2,2);hold on;
md = mean(sqrt((squeeze(sim.positions(:,:,1))-repmat(squeeze(sim.positions(1,:,1)),size(sim.positions,1),1)).^2+(squeeze(sim.positions(:,:,2))-repmat(squeeze(sim.positions(1,:,2)),size(sim.positions,1),1)).^2),2); % mm
md(isnan(md)) = 0; % if simulation was aborted (necessary for smoothing)
md = filtfilt(ones((60),1)/(60),1,md);
plot(time_bins,md,'Color',col);
set(gca,'XLim',[0 max(time_bins)],'YLim',[0 0.2],'TickDir','out');
xlabel('DIV'); ylabel('migration distance [mm]');

% CI
subplot(3,2,3);hold on;
sim.CI(isnan(sim.CI)) = 0; % set nan to 0 if simulation was aborted (necessary for smoothing)
plot(time_bins,filtfilt(ones(60,1)/60,1,sim.CI),'Color',col); % smoothen
plot(time_bins,time_bins*0+1,'Color','k');
set(gca,'XLim',[0 max(time_bins)],'YLim',[0.5 1.5],'TickDir','out');
xlabel('DIV'); ylabel('CI'); 

% connectivity
subplot(3,2,4);hold on;
plot(time_bins,nanmean(sim.connectivity,2),'Color',col);
set(gca,'XLim',[0 max(time_bins)],'YLim',[0 1],'TickDir','out');
xlabel('DIV'); ylabel('connectivity');

% input
subplot(3,2,5);hold on;
plot(time_bins,nanmean(sim.input,2),'Color',col);
set(gca,'XLim',[0 max(time_bins)],'YLim',[0 1],'TickDir','out');
xlabel('DIV'); ylabel('input');

% activity
subplot(3,2,6);hold on;
plot(time_bins,nanmean(sim.activity,2),'Color',col);
set(gca,'XLim',[0 max(time_bins)],'YLim',[0 1],'TickDir','out');
xlabel('DIV'); ylabel('activity');

%% neurite field size distribution
figure, hold on,
dist = histc(sim.radius(dd,:),[0:.0025:.2]); % adjust bins to radius distribution
dist = dist./sum(dist);
plot([0:.0025:.2],dist,'Color',[0 0 0])
xlabel('radius');
ylabel('fraction');
set(gca,'TickDir','out');

%% Neurite density plot
ds =.001; % pixel resolution of the image (mm)
ax = 0:ds:network_size(1);
ay = 0:ds:network_size(2);
Ax = meshgrid(ax,ay);
Ay = meshgrid(ay,ax)';
A = zeros(length(ax),length(ay));
for ii=1:n_neurons
    dx = abs(neurons.position(ii,1)-Ax); dx = min(dx,network_size(1)-dx);
    dy = abs(neurons.position(ii,2)-Ay); dy = min(dy,network_size(2)-dy);
    d = sqrt(dx.^2+dy.^2);
    id = find(d<neurons.radius(ii));
    A(id) = A(id)+1;
end
figure, hold on;
imagesc(0:ds:1,0:ds:1,A);
colmap = [[0 0 0];parula(255)];
colormap(colmap);
caxis([0 15]);
axis equal image
ang = 0:0.01:2*pi;
xval = [repmat(neurons.position(:,1),1,length(ang))+repmat(cos(ang),n_neurons,1).*repmat(neurons.radius,1,length(ang)),nan(n_neurons,1)]';
yval = [repmat(neurons.position(:,2),1,length(ang))+repmat(sin(ang),n_neurons,1).*repmat(neurons.radius,1,length(ang)),nan(n_neurons,1)]';
xval = mod(xval+network_size(1),network_size(1));
yval = mod(yval+network_size(1),network_size(1));
id = find(sqrt(diff(xval(:)).^2+diff(yval(:)).^2)>(2*pi*sqrt(prod(network_size)))/length(ang));
xval(id)= nan;
yval(id)= nan;
plot(xval,yval,'k-');
plot(neurons.position(:,1),neurons.position(:,2),'ko','MarkerFaceColor','r');
cb = colorbar; set(get(cb,'YLabel'),'String','neurite overlap','rotation',-90,'verticalalignment','bottom');
set(gca,'TickDir','out'); xlabel('x [\mum]'); ylabel('y [\mum]')

%% create and save movie from sim structure
vidObj = VideoWriter('network_development','MPEG-4');
vidObj.Quality = 100;
vidObj.FrameRate = 20;
open(vidObj);

figure, hold on;
ds =.001;
ax = 0:ds:network_size(1);
ay = 0:ds:network_size(2);
Ax = meshgrid(ax,ay);
Ay = meshgrid(ay,ax)';
A = zeros(length(ax),length(ay));
ha_img = imagesc(0:ds:network_size(1),0:ds:network_size(2),A);
colmap = [[0 0 0];parula(255)];
colormap(colmap);
caxis([0 15]);
axis equal image
col = [[1 0 0];[0 0 0]];
ang = 0:0.01:2*pi;
xval = [repmat(squeeze(sim.positions(1,:,1))',1,length(ang))+repmat(cos(ang),n_neurons,1).*repmat(sim.radius(1,:)',1,length(ang)),nan(n_neurons,1)]';
yval = [repmat(squeeze(sim.positions(1,:,2))',1,length(ang))+repmat(sin(ang),n_neurons,1).*repmat(sim.radius(1,:)',1,length(ang)),nan(n_neurons,1)]';
xval = mod(xval+network_size(1),network_size(1));
yval = mod(yval+network_size(1),network_size(1));
id = find(sqrt(diff(xval(:)).^2+diff(yval(:)).^2)>(2*pi*sqrt(prod(network_size)))/length(ang));
xval(id)= nan;
yval(id)= nan;
ha_circle = plot(xval(:),yval(:),'-','Color',[1 1 1]);
ha_position  = plot(sim.positions(1,:,1),sim.positions(1,:,2),'wo','MarkerFaceColor','r');
axis square off
set(gcf,'position',[100 100 1000 1000])
% text(network_size(2)/40,network_size(1)/40,[char(169) ' Okujeni 2019'],'FontWeight','bold','FontSize',20,'HorizontalAlignment','left','Color','w');
ha_div = text(network_size(2)/40,network_size(1)-network_size(1)/40,[sprintf('%2.0f',0), ' DIV'],'FontWeight','bold','FontSize',20,'HorizontalAlignment','left','interp','none','Color','w');

speedup = 60; % simulation steps to skip
max_DIV = 50; % maximal time (days)
for dd = 1:speedup:max_DIV*24*60/dt+1
    set(ha_position,'xdata',sim.positions(dd,:,1),'ydata',sim.positions(dd,:,2));
    xval = [repmat(squeeze(sim.positions(dd,:,1))',1,length(ang))+repmat(cos(ang),n_neurons,1).*repmat(sim.radius(dd,:)',1,length(ang)),nan(n_neurons,1)]';
    yval = [repmat(squeeze(sim.positions(dd,:,2))',1,length(ang))+repmat(sin(ang),n_neurons,1).*repmat(sim.radius(dd,:)',1,length(ang)),nan(n_neurons,1)]';
    xval = mod(xval+network_size(1),network_size(1));
    yval = mod(yval+network_size(1),network_size(1));
    id = find(sqrt(diff(xval(:)).^2+diff(yval(:)).^2)>(2*pi*sqrt(prod(network_size)))/length(ang));
    xval(id)= nan;
    yval(id)= nan;
    set(ha_circle,'XData',xval(:),'YData',yval(:));
    set(ha_div,'String',strtrim([sprintf('%12.0f',floor(time_bins(dd))), ' DIV ']));
    A = zeros(length(ax),length(ay));
    for ii=1:n_neurons
        dx = abs(sim.positions(dd,ii,1)-Ax); dx = min(dx,network_size(1)-dx);
        dy = abs(sim.positions(dd,ii,2)-Ay); dy = min(dy,network_size(2)-dy);
        d = sqrt(dx.^2+dy.^2);
        id = find(d<sim.radius(dd,ii));
        A(id) = A(id)+1;
    end
    set(ha_img,'CData',A);
    % Write each frame to the file.
    currFrame = getframe;
    writeVideo(vidObj,currFrame);
    disp(time_bins(dd));
end
close(vidObj);
%% evolution of giant component over time
res = 24*60/dt;  % resolution in time steps (1 day)
divbins = 1:res:length(time_bins);
gc = zeros(size(divbins));
for ii=1:length(divbins),
    distmat = getDistanceTorus(squeeze(sim.positions(divbins(ii),:,:)),network_size);
    overlap = getCirclesIntersection(sim.radius(divbins(ii),:)',distmat);
    gc(ii) = getGiantcomponent(overlap>0);
end
gc = gc/size(overlap,1);
%
figure, hold on;
plot((divbins-1)/res,gc,'Color','r');
xlabel('DIV'); ylabel('giant component');
axis square
set(gca,'TickDir','out');
set(gca,'YLim',[0 1.001]);

%% giant component analysis for a given configuration
reps = 100; % number of repetitions
thetas = [0:5:100];
GC = nan(length(thetas),reps);
NS = nan(length(thetas),reps);
C = neurons.connectivity>0;
C(isnan(C)) = 0;
for ii=1:length(thetas), % fraction
    rm_neuron = ceil(size(C,1)/100*thetas(ii))+1;
    for jj = 1:reps
        id = randperm(size(C,1));
        E = C(id(rm_neuron:end),id(rm_neuron:end));
        if ~isempty(E),
            GC(ii,jj)=  getGiantcomponent(E);
        else
            GC(ii,jj)=nan;
        end
        NS(ii,jj)= size(E,1);
    end
end

figure, hold on;
rGC = nanmean(GC,2)./nanmean(NS,2);
plot(thetas(1:end-1),rGC(1:end-1),'Color','r');
xlabel('fraction'); ylabel('giant component');
axis square
set(gca,'TickDir','out');

if rGC(1)<1 % normalize for not fully connected networks to largest initial component
    rGC = rGC/rGC(1);
    plot(thetas(1:end-1),rGC(1:end-1),'r:');
end
%% Louvain modularity
C = neurons.connectivity;
C(find(eye(size(C)))) = 0;
[M, Q]     = community_louvain(C,1);
disp(Q);